/*******************************************************************************
 * Copyright (c) 2000, 2006 QNX Software Systems and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     QNX Software Systems - Initial API and implementation
 *******************************************************************************/
package org.eclipse.cdt.debug.mi.core.output;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * GDB/MI thread list parsing.
~"\n"
~"     2 Thread 2049 (LWP 29354)  "
~"* 1 Thread 1024 (LWP 29353)  "

 */
public class CLIInfoThreadsInfo extends MIInfo {

	protected String[] threadNames;
	protected int[] threadIds;
	protected int currentThreadId;

	public CLIInfoThreadsInfo(MIOutput out) {
		super(out);
		parse();
	}

	public int[] getThreadIds() {
		return threadIds;
	}

	public String[] getThreadNames() {
		return threadNames;
	}

	public int getCurrentThread() {
		return currentThreadId;
	}

	protected void parse() {
		List<XThread> aList = new ArrayList<XThread>();
		if (isDone()) {
			MIOutput out = getMIOutput();
			MIOOBRecord[] oobs = out.getMIOOBRecords();
			for (int i = 0; i < oobs.length; i++) {
				if (oobs[i] instanceof MIConsoleStreamOutput) {
					MIStreamRecord cons = (MIStreamRecord) oobs[i];
					String str = cons.getString();
					// We are interested in finding the current thread
					parseThreadInfo(str.trim(), aList);
				}
			}
		}
		
		Collections.sort(aList);
		
		threadIds = new int[aList.size()];
		threadNames = new String[aList.size()];
		for (int i = 0; i < aList.size(); i++) {
			XThread t = aList.get(i);
			threadIds[i] = t.id;
			threadNames[i] = t.toString();
		}
		//Arrays.sort(threadIds);
	}

	private final static int GROUP_ID = 1;
	private final static int GROUP_CORE_SYMBOL = 2;
	private final static int GROUP_CORE = 3;
	private final static int GROUP_THREAD = 4;
//	private final static Pattern threadPattern = Pattern.compile("\\*?\\s*([0-9]+)\\s*Node ([0-9]+) XCore ([0-9]+) Thread ([0-9]+)");
	private final static Pattern threadPattern = Pattern.compile("\\*?\\s*([0-9]+)\\s*(stdcore|xcore)\\[([0-9]+)\\]\\s*hwthread\\s*([0-9]+)");
	
	protected void parseThreadInfo(String str, List<XThread> aList) {

		if (str.length() > 0) {

			Matcher matcher = threadPattern.matcher(str);
			if (!matcher.find()){
				return;
			}
						
			boolean isCurrentThread = false;
			// Discover the current thread
			if (str.charAt(0) == '*') {
				isCurrentThread = true;
				str = str.substring(1).trim();
			}

			try {
				int id = Integer.parseInt(matcher.group(GROUP_ID));
				String sym = matcher.group(GROUP_CORE_SYMBOL);
				int core = Integer.parseInt(matcher.group(GROUP_CORE));
				int thread = Integer.parseInt(matcher.group(GROUP_THREAD));
			
				aList.add(new XThread(sym, thread, core, id));
				if (isCurrentThread) {
					currentThreadId = id;
				}
			} catch (NumberFormatException e) {
				e.printStackTrace();
			}
		}
	}
}

class XThread implements Comparable<XThread>{
	
	public int threadID;
	public int coreID;
	public int id;
	public String coreSymbol;
	public XThread(String coreSymbol, int threadID, int coreID, int id) {
		super();
		this.coreSymbol = coreSymbol;
		this.threadID = threadID;
		this.coreID = coreID;
		this.id = id;
	}
	public int compareTo(XThread arg0) {
		return id - arg0.id;
	}
	public String toString(){
		return coreSymbol + "[" + coreID + "] thread " + threadID;
	}
}
